# Created by: Guillermo Gonzalez
# Updated with comments/fixes of Jamie Teer, Scott Cukras and Miguel Betin
# Everything is going to be included (1 based)
# Variant files are already 1 based (nothing changes)
# Bed files : add 1 to the start position

# Import libraries:
import os
import sys
sys.path.insert(1, os.path.join(sys.path[0], '..'))
import time
from share.classes.ConfigReader import ConfigReader
from pymongo import MongoClient
import logging
import datetime

now = datetime.datetime.now()
current_date = now.utcnow()

config_reader = ConfigReader(os.path.join(sys.path[0], '../resources/config/config.yaml'))
config_reader.read_config()

# --------------------------------------------REQUIRED VARIABLES:---------------------------------------------------
# Cohort info:
cohort_id                = config_reader.yaml['cohort']['id']
cohort_uploaded_by       = config_reader.yaml['cohort']['uploaded_by']
project_name             = config_reader.yaml['cohort']['project_name']
data_set_name            = config_reader.yaml['cohort']['data_set_name']
cohort_description       = config_reader.yaml['cohort']['description']
target_regions_file_path = config_reader.yaml['cohort']['target_regions_file_path']
variants_file_path       = config_reader.yaml['cohort']['variants_file_path']
missing_data_files_path  = config_reader.yaml['cohort']['missing_data_files_path']

# General Variables:
path = config_reader.yaml['cohort']['path']
replace_existing_cohort = config_reader.yaml['cohort']['replace_existing_cohort']
# ----------------------------------------------------------------------------------------------------------------------


chromosome_list = ['1', '2', '3', '4', '5', '6', '7', '8', '9',
                   '10', '11', '12', '13', '14', '15', '16', '17',
                   '18', '19', '20', '21', '22', 'X', 'Y', 'MT']

total_number_of_target_regions = 0
total_number_of_missing_data_regions = 0
total_number_of_positions = 0

# Functions:

def get_dict_from_list(list, key, value):
    for d in list:
        if d.get(key) == value:
            return d
    return None

def is_position_in_target_region(position, target_regions):

    # min_target_regions_position = cohort.get('min_target_regions_position')
    # max_target_regions_position = cohort.get('max_target_regions_position')

    min_target_regions_position = min(region.get('start') for region in target_regions)
    max_target_regions_position = max(region.get('end') for region in target_regions)

    # tally = 0
    if position < min_target_regions_position or position > max_target_regions_position:
        return False
    for region in target_regions:
        # tally += 1
        start = region.get('start')
        end = region.get('end')
        # Include start position since .bed files were added 1 to the start position to be 1 based:
        if start <= position <= end:
            # if start < position <= end:
            # print('tally :', tally)
            return True
    # print('tally :', tally)
    return False

# Loggers:

# General Logger:
logging.basicConfig(filename=path + 'nsmLoader-' + cohort_id + '-' + now.strftime("%Y-%m-%d")+'.log',
                    level=logging.DEBUG, format='%(asctime)s %(message)s',
                    datefmt='%m/%d/%Y %I:%M:%S %p', filemode='w')

# define a Handler which writes INFO messages or higher to the sys.stderr
console = logging.StreamHandler()
console.setLevel(logging.INFO)
# set a format which is simpler for console use
formatter = logging.Formatter('%(name)-12s: %(levelname)-8s %(message)s')
# tell the handler to use this format
console.setFormatter(formatter)
# add the handler to the root logger
logging.getLogger('').addHandler(console)

# Now, we can log to the root logger, or any other logger. First the root...
# logging.info('Test message')

# Connect to MongoDB:
client = MongoClient(config_reader.yaml['mongodb']['host'])
# adminDb = client.admin
adminDb = client[config_reader.yaml['mongodb']['authSource']]
adminDb.authenticate(config_reader.yaml['mongodb']['user'], config_reader.yaml['mongodb']['passwd'], mechanism='SCRAM-SHA-1')

# db = client.negativeStorageModel
db = client[config_reader.yaml['mongodb']['nsm_database']]

total_start_time = time.time()

cohort = {
    "_id": cohort_id,
    "uploaded_by": cohort_uploaded_by,
    "project_name": project_name,
    "uploaded_date": current_date,
    "data_set_name": data_set_name,
    "description": cohort_description,
    'target_regions_file_path': target_regions_file_path,
    'min_target_regions_position': 0,
    'max_target_regions_position': 0,
    'min_variants_position': 0,
    'max_variants_position': 0,
    'min_missing_data_position': 0,
    'max_missing_data_position': 0,
}

db_cohort = db.cohorts.find_one({"_id": cohort_id})

if db_cohort is not None:  # If cohort exists
    logging.info('The cohort ' + cohort_id + ' already exists')
    if replace_existing_cohort:
        # Delete cohort data from MongoDB
        db.cohorts.delete_one({"_id": cohort_id})
        db.target_regions.delete_many({"cohort_id": cohort_id})
        db.missing_data_regions.delete_many({"cohort_id": cohort_id})
        db.positions.delete_many({"cohort_id": cohort_id})
        db.batches.delete_many({"cohort_id": cohort_id})
        logging.info('Previous cohort ' + cohort_id + ' data deleted')
        # Uncomment next line to delete the existing cohort:
        # exit(0)
        # Insert cohort into MongoDB:
        db_cohort_id = db.cohorts.insert_one(cohort).inserted_id
        db_cohort = db.cohorts.find_one({"_id": db_cohort_id})
        logging.info('New Cohort ' + cohort_id + ' created')
    else:
        # logging.info('Not replacing existing cohort:' + cohort_id)
        logging.info('Adding data to existing cohort: ' + cohort_id)
    #     exit(0)
else:  # If cohort does not exist
    replace_existing_cohort = True
    # Insert cohort into MongoDB:
    db_cohort_id = db.cohorts.insert_one(cohort).inserted_id
    db_cohort = db.cohorts.find_one({"_id": db_cohort_id})
    logging.info('New Cohort ' + cohort_id + ' created')

# else:
#     raise Exception('The cohort already exists')
# logging.error(db_cohort.get('_id'))

# Get the list of all samples from the database:
all_samples_list = db.positions.distinct('samples.sample')

# Set of samples that were already in the database and therefore not loaded:
samples_already_loaded_set = set()

# Set of processed samples:
processed_samples_set = set()

# Get the sample list that have a dash (-) in the alt column:

samples_with_dash_in_alt_column = []
try:
    samples_with_dash_in_alt_column_file = open(path + 'variants/samples_with_dash_in_alt_column')
    line = samples_with_dash_in_alt_column_file.readline().strip()
    while line:
        samples_with_dash_in_alt_column.append(line)
        line = samples_with_dash_in_alt_column_file.readline().strip()
    samples_with_dash_in_alt_column_file.close()
except FileNotFoundError:
    logging.error('The prep_variants_files.sh script has not been run')
    exit(0)

tmp_batch = db.batches.find_one({'cohort_id': cohort_id})
if tmp_batch is not None:  # if batch exists
    max_batch_id = db.batches.find_one({'cohort_id': cohort_id}, sort=[('id', -1)])['id']
    batch_id = max_batch_id + 1
else:
    batch_id = 1

logging.info('------------------------------------------------------ ' + cohort_id + ' Cohort ------------------------------------------------------')
logging.info('')
logging.info('')
logging.info('')

for chromosome in chromosome_list:

    start_time = time.time()

    logging.info('----------------------------------- Chromosome ' + chromosome + '-----------------------------------')
    logging.info('')

    # Target Regions:
    target_regions = []
    if replace_existing_cohort:
        try:
            target_regions_file = open(path + 'target_regions/target_regions-chr-' + chromosome + '.bed')
        except FileNotFoundError:
            logging.error('There is no target regions file for chromosome ' + chromosome)
            continue
        line = target_regions_file.readline()
        elements = line.split('\t')
        # Min position:
        start_position = int(elements[1])
        min_position = start_position
        tally = 0
        previous_start_position = 0
        max_position = 0

        while line:
            tally += 1
            elements = line.split('\t')
            # chromosome = elements[0]
            start_position = int(elements[1])
            end_position = int(elements[2])

            # validate if positions are sorted:
            # if previous_start_position > start_position:
            #     raise Exception('The positions are not sorted:', start_position)
            # previous_start_position = start_position

            # Check for max position:
            if end_position > max_position:
                max_position = end_position

            target_regions += [
                {
                    'cohort_id': cohort_id,
                    'batch_id': batch_id,
                    'chrom': chromosome,
                    # Add 1 to the start position to be 1 based:
                    'start': start_position + 1,
                    'end': end_position,
                }
            ]
            line = target_regions_file.readline()
        target_regions_file.close()
        # update cohort dictionary:
        cohort.update({'min_target_regions_position': min_position})
        cohort.update({'max_target_regions_position': max_position})
        # update cohort in db:
        db.cohorts.update_one(
            {
                '_id': cohort_id
            },
            {
                '$set': {
                    'min_target_regions_position': min_position,
                    'max_target_regions_position': max_position
                }
            }
        )

        logging.info('Number of lines in target_regions file: ' + str(tally))

        # Insert cohort into MongoDB:
        db.target_regions.insert_many(target_regions)
        total_number_of_target_regions += len(target_regions)
        logging.info(str(len(target_regions)) + ' target regions inserted successfully')
    else:  # not replace_existing_cohort
        logging.info("Getting target_regions from database...")
        cohort.update({'min_target_regions_position': db_cohort.get('min_target_regions_position')})
        cohort.update({'max_target_regions_position': db_cohort.get('max_target_regions_position')})
        db_target_regions = db.target_regions.find({"cohort_id": cohort_id, "chrom": chromosome})
        target_regions = list(db_target_regions)
        logging.info('Number of documents in target_regions db collection: ' + str(db_target_regions.count()))
    # logging.info('Target regions :' + str(target_regions))
        # exit(0)
    # ----------------------------------------------------------------------------------------------------------

    # Missing Data:
    try:
        missing_data_file = open(path + 'missing_data/missing_data-chr-' + chromosome + '.txt')

        line = missing_data_file.readline()
        elements = line.split('\t')
        # Min position:
        start_position = int(elements[2])
        min_position = start_position
        tally = 0
        previous_position = 0
        previous_sample = elements[0]
        max_position = 0

        missing_data_regions = []

        while line:
            tally += 1
            elements = line.split('\t')
            sample = elements[0]

            # Check if the sample exists in the list of samples with dashes in the alt column; if so, continue with next line:
            if sample in samples_with_dash_in_alt_column:
                # logging.error('The sample ' + sample + ' has a dash (-) in at least one of its variants. It WAS NOT processed')
                line = missing_data_file.readline()
                continue

            # Check if the sample already exists in the database; if so, continue with next line:
            if sample in all_samples_list:
                samples_already_loaded_set.add(sample)
                # logging.error('The sample ' + sample + ' already exists in the database. It WAS NOT processed')
                line = missing_data_file.readline()
                continue

            # chromosome = elements[1]
            start_position = int(elements[2])  # 0 based (not included)
            end_position = int(elements[3])  # 1 based (included)

            # validate if positions are sorted:
            # if previous_position >= start_position and previous_sample == sample:
                # raise Exception('The positions are not sorted or are not merged', start_position)
                # logging.error('The positions are not sorted', start_position)
            # previous_position = start_position
            # previous_sample = sample

            # Check for max position:
            if end_position > max_position:
                max_position = end_position

            # Check if the missing data region already exists in the database, if adding data to an existing cohort:
            if not replace_existing_cohort:
                missing_data_region_exists_in_db = db.missing_data_regions.find_one({"cohort_id": cohort_id, "chrom": chromosome, "sample": sample, "start": start_position + 1, "end": end_position})
                if missing_data_region_exists_in_db:
                    logging.info("The missing data region (sample ,chrom, sample, start, end) (" + sample + ", " + chromosome + ", " + sample + ", " + str(start_position + 1) + ", " + str(end_position) + ") already exists in the database for the " + str(cohort_id) + " cochort")
                    line = missing_data_file.readline()
                    continue

            missing_data_regions += [
                {
                    'cohort_id': cohort_id,
                    'batch_id': batch_id,
                    'chrom': chromosome,
                    'sample': sample,
                    # Add 1 to the start position to be 1 based:
                    'start': start_position + 1,
                    'end': end_position
                }
            ]
            processed_samples_set.add(sample)
            line = missing_data_file.readline()
        missing_data_file.close()
        # update cohort dictionary:
        cohort.update({'min_missing_data_position': min_position})
        cohort.update({'max_missing_data_position': max_position})
        # update cohort in db:
        db.cohorts.update_one(
            {
                '_id': cohort_id
            },
            {
                '$set': {
                    'min_missing_data_position': min_position,
                    'max_missing_data_position': max_position
                }
            }
        )

        logging.info('----------------------------------------------------------')
        logging.info('Number of lines in missing_data_file: ' + str(tally))

        if len(missing_data_regions) > 0:
            db.missing_data_regions.insert_many(missing_data_regions)
            logging.info(str(len(missing_data_regions)) + ' missing data regions inserted successfully')
        else:
            logging.info('No missing data regions were inserted')
        total_number_of_missing_data_regions += len(missing_data_regions)

    except FileNotFoundError:
        logging.error('There is no missing data file for chromosome ' + chromosome)

    # -----------------------------------------------------------------------------------------------------------------
    # Variants:
    try:
        variants_file = open(path + 'variants/variants-chr-' + chromosome + '.tsv')
    except FileNotFoundError:
        logging.error('There is no variants file for chromosome ' + chromosome)
        logging.info('')
        logging.info('')
        continue
    line = variants_file.readline()
    elements = line.split('\t')
    # chromosome = elements[9]

    # Min position:
    position = int(elements[10])
    min_position = position

    # get rid of the header:
    # line = variants_file.readline()
    # logging.info(line)
    # ['id', 'sample', 'line', 'function', 'gene', 'symbol', 'exon', 'dna', 'protein',
    #  'chrom', 'pos', 'pos2', 'ref', 'alt','vid', 'qual', 'filter', 'info', 'format',
    #  'genotype', 'mutationtype', 'g1000v', 'coverage', 'ncoverage', 'familyid\n']

    tally = 0
    positionSet = set()
    previous_position = 0
    max_position = 0

    positions = []
    db_positions = []

    if not replace_existing_cohort:
        logging.info("Getting positions from database...")
        # db_positions = db.positions.find({"cohort_id": cohort_id, "chrom": chromosome}, {'_id': 0})
        db_positions = db.positions.find({"cohort_id": cohort_id, "chrom": chromosome})
        db_positions = list(db_positions)

    while line:
        tally += 1
        elements = line.split('\t')
        sample = elements[1]

        # Check if the sample exists in the list of samples with dashes in the alt column; if so, continue with next line:
        if sample in samples_with_dash_in_alt_column:
            # logging.error('The sample ' + sample + ' has a dash (-) in at least one of its variants. It WAS NOT processed')
            line = variants_file.readline()
            continue

        # Check if the sample already exists in the database; if so, continue with next line:
        if sample in all_samples_list:
            samples_already_loaded_set.add(sample)
            # logging.error('The sample ' + sample + ' already exists in the database. It WAS NOT processed')
            line = variants_file.readline()
            continue

        function = elements[3]  # impact
        gene = elements[4]  # gene id
        symbol = elements[5]  # gene name
        exon = elements[6]
        protein = elements[8]
        chromosome = elements[9]
        position = int(elements[10])
        ref = elements[12]
        alt = elements[13]
        # genotype = elements[19]
        genotype = elements[19] if elements[19] != '.' else '0/1'

        alt_elements = alt.split(',')
        protein_elements = protein.split(':')
        function_elements = []

        genotype_elements = genotype.split('/')
        base_genotype_elements = [ref] + alt_elements
        first_genotype_element = genotype_elements[0]
        second_genotype_element = genotype_elements[1]
        base_genotype = base_genotype_elements[int(first_genotype_element)] + '_' + base_genotype_elements[
            int(second_genotype_element)]

        '''
        # In case of allow uploading existing samples:
        # Check if the combination (chromosome, position, alt, ref) already exists in the database, if adding data to an existing cohort:
        if not replace_existing_cohort:
            variant_exists_in_db = db.positions.find_one({"cohort_id": cohort_id, "chrom": chromosome, "pos": position, "samples": {"$elemMatch": {"sample": sample, "genes.proteins": {"$elemMatch": {"ref": ref, "alt": alt}}}}})
            if variant_exists_in_db:
                logging.info("The variant (sample, chromosome, position, alt, ref) (" + sample + ", " + chromosome + ", " + str(position) + ", " + alt + ", " + ref + ") already exists in the database for the " + str(cohort_id) + " cochort")
                line = variants_file.readline()
                continue
        '''
        # Check if position is in the targeted subset
        if not is_position_in_target_region(position, target_regions):
            # inferred missing, not store
            logging.error('The position ' + str(position) + ' is not in the target regions (Sample ' + sample + ')')
            line = variants_file.readline()
            continue

        positionSet.add(position)
        # validate if positions are sorted:
        # if previous_position > position:
        #     raise Exception('The positions are not sorted')
        # previous_position = position

        # Check for max position:
        if position > max_position:
            max_position = position

        if not replace_existing_cohort:
            # Check if the position already exists in the database:
            current_position = get_dict_from_list(db_positions, 'pos', position)
            if current_position is not None:
                samples_from_current_position = current_position.get('samples')
                # Check if sample already exists in position:
                if get_dict_from_list(samples_from_current_position, 'sample', sample) is None:
                    status = 'reference' if ref == alt else 'variant'
                    if status == 'reference':
                        raise Exception('should not be stored', position)
                    samples_from_current_position += [{'sample': sample, 'status': status, 'genotype': genotype,
                                                       'base_genotype': base_genotype, 'batch_id': batch_id,
                                                       'genes': []}]
                current_sample = get_dict_from_list(samples_from_current_position, 'sample', sample)
                genes_from_current_sample = current_sample.get('genes')

                # Check if gene is empty:
                # Validation to add each empty gene as an independent item to the genes array
                if gene == '' or gene == 'NULL':
                    genes_from_current_sample += [{'gene': gene, 'symbol': symbol, 'exon': exon, 'proteins': []}]
                    current_gene = genes_from_current_sample[-1]
                else:  # Check if gene already exists in sample:
                    if get_dict_from_list(genes_from_current_sample, 'gene', gene) is None:
                        genes_from_current_sample += [{'gene': gene, 'symbol': symbol, 'exon': exon, 'proteins': []}]
                    current_gene = get_dict_from_list(genes_from_current_sample, 'gene', gene)
                proteins_from_current_gene = current_gene.get('proteins')
                # Check if protein already exists in gene:
                if get_dict_from_list(proteins_from_current_gene, 'protein', protein) is None:
                    if len(protein_elements) == 1:
                        protein_elements = []
                        for i in range(len(alt_elements)):
                            protein_elements.append(protein)
                    for i in range(len(alt_elements)):
                        function_elements.append(function)
                    for i in range(len(alt_elements)):
                        proteins_from_current_gene += [
                            {'protein': protein_elements[i], 'ref': ref, 'alt': alt_elements[i],
                             'function': function_elements[i]}]

                # update position in db:
                db.positions.update_one(
                    {
                        '_id': current_position.get('_id')
                    },
                    {
                        '$set': {
                            'samples': samples_from_current_position
                        }
                    }
                )

                line = variants_file.readline()
                continue

        # Check if position already exists in chromosome:
        if get_dict_from_list(positions, 'pos', position) is None:
            positions += [
                {
                    'cohort_id': cohort_id,
                    'batch_id': batch_id,
                    'chrom': chromosome,
                    'pos': position,
                    'samples': [],
                }
            ]
        current_position = get_dict_from_list(positions, 'pos', position)
        samples_from_current_position = current_position.get('samples')
        # Check if sample already exists in position:
        if get_dict_from_list(samples_from_current_position, 'sample', sample) is None:
            status = 'reference' if ref == alt else 'variant'
            if status == 'reference':
                raise Exception('should not be stored', position)
            samples_from_current_position += [{'sample': sample, 'status': status, 'genotype': genotype,
                                               'base_genotype': base_genotype, 'batch_id': batch_id, 'genes': []}]
        current_sample = get_dict_from_list(samples_from_current_position, 'sample', sample)
        genes_from_current_sample = current_sample.get('genes')

        # Check if gene is empty:
        # Validation to add each empty gene as an independent item to the genes array
        if gene == '' or gene == 'NULL':
            genes_from_current_sample += [{'gene': gene, 'symbol': symbol, 'exon': exon, 'proteins': []}]
            current_gene = genes_from_current_sample[-1]
        else:  # Check if gene already exists in sample:
            if get_dict_from_list(genes_from_current_sample, 'gene', gene) is None:
                genes_from_current_sample += [{'gene': gene, 'symbol': symbol, 'exon': exon, 'proteins': []}]
            current_gene = get_dict_from_list(genes_from_current_sample, 'gene', gene)
        proteins_from_current_gene = current_gene.get('proteins')
        # Check if protein already exists in gene:
        if get_dict_from_list(proteins_from_current_gene, 'protein', protein) is None:
            if len(protein_elements) == 1:
                protein_elements = []
                for i in range(len(alt_elements)):
                    protein_elements.append(protein)
            for i in range(len(alt_elements)):
                function_elements.append(function)
            for i in range(len(alt_elements)):
                proteins_from_current_gene += [{'protein': protein_elements[i], 'ref': ref, 'alt': alt_elements[i],
                                                'function': function_elements[i]}]
        processed_samples_set.add(sample)
        line = variants_file.readline()
    variants_file.close()

    # update cohort dictionary:
    cohort.update({'min_variants_position': min_position})
    cohort.update({'max_variants_position': max_position})

    # update cohort in db:
    db.cohorts.update_one(
        {
            '_id': cohort_id
        },
        {
            '$set': {
                'min_variants_position': min_position,
                'max_variants_position': max_position
            }
        }
    )

    logging.info('----------------------------------------------------------')
    logging.info('Number of lines in variants_file: ' + str(tally))
    logging.info('Number of positions in variants_file: ' + str(len(positionSet)))

    # Insert cohort into MongoDB:
    if len(positions) > 0:
        db.positions.insert_many(positions)
        logging.info(str(len(positions)) + ' positions inserted successfully')
    else:
        logging.info('No positions were inserted')
    total_number_of_positions += len(positions)

    logging.info('----------------------------------------------------------')

    end_time = time.time()
    execution_time = end_time - start_time
    logging.info('Execution time for chromosome ' + chromosome + ': ' + str(execution_time) + ' seconds')

    day = execution_time // (24 * 3600)
    execution_time = execution_time % (24 * 3600)
    hour = execution_time // 3600
    execution_time %= 3600
    minutes = execution_time // 60
    execution_time %= 60
    seconds = execution_time
    logging.info("d:h:m:s --> %d:%d:%d:%d" % (day, hour, minutes, seconds))
    logging.info('----------------------------------------------------------')
    logging.info('')
    logging.info('')
    logging.info('')

logging.info('')
logging.info('')
logging.info('')
if total_number_of_missing_data_regions == 0 and total_number_of_positions == 0:
    logging.info('No Batches were created')
else:
    batch = {
        'id': batch_id,
        'cohort_id': cohort_id,
        'uploaded_by': cohort_uploaded_by,
        'uploaded_date': current_date,
        'total_number_of_target_regions_added': total_number_of_target_regions,
        'total_number_of_missing_data_regions_added': total_number_of_missing_data_regions,
        'total_number_of_positions_added': total_number_of_positions,
        'variants_file_path': variants_file_path,
        'missing_data_files_path': missing_data_files_path
    }
    db.batches.insert_one(batch)
    logging.info('Batch ' + str(batch_id) + ' inserted successfully')


# Print info about processed samples:
processed_samples = list(processed_samples_set)
# logging.error('processed_samples:' + str(processed_samples).strip('[]'))
logging.info('')
logging.info('')
logging.info('--------------------------------------------------------------------------------------')
logging.info('SAMPLES PROCESSED (' + str(len(processed_samples)) + '):')
for processed_sample in processed_samples:
    logging.info('Sample ' + processed_sample + ' was processed successfully')
logging.info('--------------------------------------------------------------------------------------')


# Print info about samples already loaded in the database
samples_already_loaded = list(samples_already_loaded_set)
logging.info('')
logging.info('')
logging.info('--------------------------------------------------------------------------------------')
logging.error('NON-PROCESSED SAMPLES BECAUSE THEY ALREADY EXIST IN THE DB(' + str(len(samples_already_loaded)) + '):')
for sample_already_loaded in samples_already_loaded:
    logging.error('Sample ' + sample_already_loaded + ' already exists in the database and WAS NOT PROCESSED')
logging.info('--------------------------------------------------------------------------------------')

logging.info('')
logging.info('')
logging.info('--------------------------------------------------------------------------------------')
logging.error('NON-PROCESSED SAMPLES BECAUSE OF DASHES (-) IN THE ALT COLUMN(' + str(len(samples_with_dash_in_alt_column)) + '):')
for sample_with_dash_in_alt_column in samples_with_dash_in_alt_column:
    logging.error('Sample ' + sample_with_dash_in_alt_column + ' has a dash (-) in the ALT column at least in one of its variants. It WAS NOT processed')
logging.info('--------------------------------------------------------------------------------------')

# Update sample and gene lists on cohort:
sample_list = db.positions.distinct('samples.sample', {'cohort_id': cohort_id})
sample_list.sort()

gene_list = db.positions.distinct('samples.genes.symbol', {'cohort_id': cohort_id})
gene_list.sort()

db.cohorts.update_one(
        {
            '_id': cohort_id
        },
        {
            '$set': {
                'samples': sample_list,
                'genes': gene_list
            }
        }
    )
logging.info('')
logging.info('')
logging.info('The ' + cohort_id + ' has ' + str(len(sample_list)) + ' samples')
logging.info('')
logging.info('The ' + cohort_id + ' has ' + str(len(gene_list)) + ' genes')
logging.info('')
logging.info('')

total_end_time = time.time()

total_execution_time = total_end_time - total_start_time

logging.info('Total Execution time: ' + str(total_execution_time) + ' seconds')

day = total_execution_time // (24 * 3600)
total_execution_time = total_execution_time % (24 * 3600)
hour = total_execution_time // 3600
total_execution_time %= 3600
minutes = total_execution_time // 60
total_execution_time %= 60
seconds = total_execution_time
logging.info("d:h:m:s --> %d:%d:%d:%d" % (day, hour, minutes, seconds))
