import csv
import pandas as pd

class GVCFExportService:

    def __init__(self, db, version, sample_id='', gvcf_header=''):
        self.version = version
        self.sample_id = sample_id
        self.gvcf_header = gvcf_header
        self.combined_gvcf = pd.DataFrame([], columns=[])

    def set_version(self, version):
        self.version = version

    def set_sample_id(self, sample_id):
        self.sample_id = sample_id

    def convert_to_oracle_sample_id_format(self):
        ary = self.sample_id.split('-')
        fname = ''

        for x in ary:
            fname += x
            
        self.sample_id = '00000' + fname

    def set_gvcf_header(self, gvcf_header):
        self.gvcf_header = gvcf_header

    def _append_gvcf_header(self, header, outdir):
        with open(outdir + self.sample_id + '.gvcf', 'w') as file:
            for line in self.gvcf_header.split('\\n'):
                file.write(line)
                file.write('\n')

    def combine_chrom_gvcf(self, chrom_gvcfs, gvcf_column_headers):
        print('Concatenating chromosome gVCFs')
        combined = pd.DataFrame([], columns=gvcf_column_headers)
        for i, gvcf in enumerate(chrom_gvcfs):
            print('Appending gVCF file: %s' % (i + 1))
            combined = combined.append([gvcf])
        
        self.combined_gvcf = combined

    def save_gvcf(self, outdir, gvcf_column_headers):
        print('Saving gVCF to disk...')
        if outdir[-1] != '/':
            outdir += '/'

        if '~' in outdir:
            outdir = os.path.expanduser(outdir)

        self._append_gvcf_header(gvcf_column_headers, outdir)

        with open(outdir + self.sample_id + '.gvcf', 'a') as f:
            self.combined_gvcf.to_csv(f, sep='\t',
            index_label = False, index=False, header=True, encoding='utf-8')

