# script to read discrepant gVCF file and determine error type

import os
import sys
sys.path.insert(1, os.path.join(sys.path[0], '../..'))
import yaml
import argparse
import pandas as pd
from share.classes.ConfigReader import ConfigReader

parser = argparse.ArgumentParser(description='script to read discrepant gVCF ' \
	'file and determine error type')
parser.add_argument('-i', '--input', required=True, help='gVCF file')

args = parser.parse_args()

config_reader = ConfigReader(os.path.join(sys.path[0], '../../resources/config/config.yaml'))
config_reader.read_config()

GVCF_HEADER_ROWS_TO_SKIP = 19

################################################################################
################################################################################
################################################################################
# functions

def convert_id_to_gvcf_id(sampleid):
	return '00000' + sampleid.replace('-', '')

def read_gvcf(gvcf):
	return pd.read_csv(gvcf, skiprows=GVCF_HEADER_ROWS_TO_SKIP, 
           		header=GVCF_HEADER_ROWS_TO_SKIP+1, sep='\t',
                dtype={ '#CHROM': str, 'POS': int, 'ID': str, 'REF': str, 'ALT': str, 'QUAL': str, 'FILTER': str, 'INFO': str, 'FORMAT': str, 'SL248569': str })

def get_pos_of_ref_length_greater_than_two(tdf):
	return tdf.index[tdf['REF'].str.len() > 1].tolist()

################################################################################
################################################################################
################################################################################
# function main()

df = read_gvcf(args.input)

indices_to_search = get_pos_of_ref_length_greater_than_two(df)

for row in indices_to_search:

	ref_length = len(df.iloc[row,:]['REF'])
	ref_start  = df.iloc[row,:]['POS']

	if ref_length > 1:
		
		current_pos = ref_start

		i = 0
		while i in range(0, len(df.index[df['POS'] == ref_start].tolist())):

			current_row = df.index[df['POS'] == ref_start].tolist()[i] + 1

			if len(df.index[df['POS'] == ref_start].tolist()) > 1:
				ref_length = len(df.iloc[current_row+i,:]['REF'])
				ref_start  = df.iloc[current_row+i,:]['POS']
				i = 0

			while df.iloc[current_row,:]['POS'] < (ref_start + ref_length):

				if df.iloc[current_row,:]['REF'] != 'X' and df.iloc[current_row,:]['REF'] != '.':
					print('Variant overlap detected @ %s:%s' % (df.iloc[current_row,:]['#CHROM'], current_pos))
					print('Invalid gVCF')
					sys.exit(1)

				current_row += 1
				current_pos = df.iloc[current_row,:]['POS']

			i += 1



