import os
import glob
import argparse
import pandas as pd

parser = argparse.ArgumentParser(description='Check gVCF stdout to determine if they ran to completion')
parser.add_argument('-i', '--indir', help='Input directory where stdout files are located', required=True)
parser.add_argument('-j', '--jobname', help='Base job name of stdout files from gVCF exporter', required=True)
parser.add_argument('-o', '--outdir', help='Output directory', required=True)

args = parser.parse_args()

if args.outdir[-1] == '/':
	args.outdir = args.outdir[:-1]

OUTPUT_FILENAME = 'gvcf_reruns.txt'

################################################################################
################################################################################
################################################################################

output_ary = []

df = pd.DataFrame([])

missing_data = False

check_list = {
	'sample': { 'str': 'Sample:', 'count': 0, 'total': 1},
	'concat': { 'str': 'Concatenating chromosome gVCFs', 'count': 0, 'total': 1 },
	'appending': { 'str': 'Appending gVCF file:', 'count': 0, 'total': 25 },
	'saving': { 'str': 'Saving gVCF to disk...', 'count': 0, 'total': 1 },
	'done': { 'str': 'Done', 'count': 0, 'total': 1 },
	'generating': { 'str': 'Generating gVCF chr:', 'count': 0, 'total': 25 },
	'created': { 'str': 'gVCF created', 'count': 0, 'total': 25 },
}

files = glob.glob(os.path.join(args.indir, args.jobname + '.o*'))
if len(files) < 1:
	files = glob.glob(os.path.join(args.indir, args.jobname + '_server_*'))

for file in files:
	
	for key in check_list:
		check_list[key]['count'] = 0

	lines = []
	passthru = [ lines.append(x.strip('\n')) for x in open(file) ]

	for line in lines:

		if check_list['sample']['str'] in line:
			sample_name = line.replace('Sample: ', '')
			check_list['sample']['count'] += 1

		if check_list['concat']['str'] in line:
			check_list['concat']['count'] += 1

		if check_list['appending']['str'] in line:
			check_list['appending']['count'] += 1

		if check_list['saving']['str'] in line:
			check_list['saving']['count'] += 1

		if check_list['done']['str'] in line:
			check_list['done']['count'] += 1

		if check_list['generating']['str'] in line:
			check_list['generating']['count'] += 1

		if check_list['created']['str'] in line:
			check_list['created']['count'] += 1


	for key in check_list:

		if check_list[key]['count'] != check_list[key]['total']:
			missing_data = True

		if missing_data:
			output_ary.append(sample_name)
			missing_data = False
			break


for sample in output_ary:
	tdf = pd.DataFrame([sample])
	df = df.append(tdf)

OUTPUT_FILE_PTH = args.outdir + '/' + OUTPUT_FILENAME

if os.path.isfile(OUTPUT_FILE_PTH):
	os.remove(OUTPUT_FILE_PTH)

if len(df) > 0:
	df.to_csv(OUTPUT_FILE_PTH, header=False, index=False)






