# Script to print general runtime statistics for exportnsm.py jobs
# This script will all create output file failed_jobs.txt
#  - output contains failed stderr file names
#  - script will print to console samples that exceeded walltime

import os
import argparse
import statistics

#############################################################
# directory where cluster stderr and stdout files are located
#############################################################

parser = argparse.ArgumentParser(description='Create runtime stats for gVCF Exporter and find sample IDs of failed jobs.')
parser.add_argument('-i', '--indir', help='Directory with cluster stdout/stderr files')
parser.add_argument('-o', '--outdir', help='Output directory')

args = parser.parse_args()

if args.indir[-1] == '/':
	INDIR = args.indir[:-1]
else:
	INDIR = args.indir

if args.outdir[-1] == '/':
	OUTDIR = args.outdir[:-1]
else:
	OUTDIR = args.outdir

if '~' in OUTDIR:
	OUTDIR = os.path.expanduser(args.outdir)

#INDIR = '/share/m2gen/bms/CS875/work/cs875_somatic/work_sc/gvcf_finals/std'
FILES = os.listdir(INDIR)

################################################################################
################################################################################
################################################################################
# functions

def open_and_get_runtime(file):
	line_ary = []

	with open(INDIR + '/' + file, 'r') as f:
		for line in f:
			line_ary.append(line)

	if len(line_ary) > 1 and line_ary[1].split('\t')[0] == 'real':
		min_sec_ary_r = line_ary[1].strip().split('\t')[1].split('m')
		min_sec_ary = []
		min_sec_ary.append(min_sec_ary_r[0])
		min_sec_ary.append(min_sec_ary_r[1].strip('s'))
		runtime_min = int(min_sec_ary[0]) + (float(min_sec_ary[1])/60)
		return runtime_min, None

	else:
		if len(line_ary) > 0 and 'walltime' in line_ary[0]:
			print('Walltime Error: %s' % file)
			print_walltime_error_sampleid(file)
		return -1, file

def print_walltime_error_sampleid(file):
	stdout   = file.replace('.e', '.o')
	line_ary = []

	with open(INDIR + '/' + stdout, 'r') as f:
		for line in f:
			line_ary.append(line)

	print(line_ary[0])

################################################################################
################################################################################
################################################################################
# main()

runtimes    = []
failed_jobs = []
for file in FILES:
	if '.e' in file:
		runtime, failed_job = open_and_get_runtime(file)
		if runtime > 0:
			runtimes.append(runtime)

		if failed_job is not None:
			failed_jobs.append(failed_job)

mean  = statistics.mean(runtimes)
med   = statistics.median(runtimes)
stdev = statistics.stdev(runtimes)
maxi  = max(runtimes)
mini  = min(runtimes)

print('All times in minutes')
print('mean : %s\nmed  : %s\nstdev: %s\nmax  : %s\nmin  : %s\n' % (mean, med, stdev, maxi, mini))

with open(OUTDIR + '/failed_jobs.txt', 'a') as f:
	for job in failed_jobs:
		f.write(job)
		f.write('\n')